" dictwin.vim - Dictionary window
"
" Maintainer:	Taro Muraoka <koron@tka.att.ne.jp>
" Last change:	07:38:41 29-Mar-2000.
" Commands:	<M-k><M-k>  Search word under cursor.
"		<M-k>/	    Search prompted word.
"		<M-k>c	    Close dictionary window.
"		<M-k>n	    Search next. (with last word)
"		<M-k>p	    Search previous. (with last word)
"		<M-k>w	    Jump to dictionary window (if exists)
" Require:	$VIM/tools/gene.txt
"
" URL where you can get 'gene.txt':
"		http://www.forest.impress.co.jp/library/gene95.html
"

let $e2j_dic = 'gene.txt'
let $e2j_path = $VIM . '/tools/' . $e2j_dic

if filereadable($e2j_path)
  let $dicwin_lastword = ''
  augroup Dictionary
    autocmd!
    autocmd WinEnter *$e2j_dic exe "normal! 8\<C-W>_"
    autocmd WinLeave *$e2j_dic exe "normal! 2\<C-W>_"
  augroup END
  nnoremap <M-k>/ :call OpenDictionary($e2j_path, input('WORD:'))<CR>
  nnoremap <M-k><M-k> :call OpenDictionary($e2j_path, expand('<cword>'))<CR>
  nnoremap <M-k>c :call CloseDictionary($e2j_path)<CR>
  nnoremap <M-k>n :call SearchNextDictionary($e2j_path)<CR>
  nnoremap <M-k>p :call SearchPrevDictionary($e2j_path)<CR>
  nnoremap <M-k>w :call SelectWindowByName($e2j_path)<CR>

  " 
  " CloseDictionary(dic)
  "   Close window of dictionary [a:dic].
  "
  function! CloseDictionary(dic)
    " let cur_name = bufname("%")
    if SelectWindowByName(a:dic) >= 0
      execute "normal! \<C-W>c"
      execute "normal! \<C-w>p"
    endi
    " call SelectWindowByName(cur_name)
  endfunction

  " SearchPrevDictionary(dic)
  "  Search next word in dictionary.
  "
  function! SearchPrevDictionary(dic)
    if SelectWindowByName(a:dic) < 0
      execute 'normal! :sv ' . a:dic ."\<CR>"
    endif
    let old_ignorecase = &ignorecase
    let &ignorecase = 1
    execute "normal! ?^" . $dicwin_lastword . "\\>\<CR>"
    let &ignorecase = old_ignorecase
    execute "normal! \<C-w>p"
  endfunction

  " SearchNextDictionary(dic)
  "  Search next word in dictionary.
  "
  function! SearchNextDictionary(dic)
    if SelectWindowByName(a:dic) < 0
      execute 'normal! :sv ' . a:dic ."\<CR>"
    endif
    let old_ignorecase = &ignorecase
    let &ignorecase = 1
    execute "normal! /^" . $dicwin_lastword . "\\>\<CR>"
    let &ignorecase = old_ignorecase
    execute "normal! \<C-w>p"
  endfunction

  " OpenDictionary(dic, word)
  "   Open window of dictionary [a:dic] and search a:word.
  "
  function! OpenDictionary(dic, word)
    " let cur_name = bufname("%")
    if SelectWindowByName(a:dic) < 0
      execute 'normal! :sv ' . a:dic ."\<CR>"
    endif
    let $dicwin_lastword = a:word
    let old_ignorecase = &ignorecase
    let &ignorecase = 1
    execute "normal! gg/^" . a:word . "\\>\<CR>"
    let &ignorecase = old_ignorecase
    execute "normal! \<C-w>p"
    " call SelectWindowByName(cur_name)
  endfunction

  "
  " SelectWindowByName(name)
  "   Acitvate selected window by a:name.
  "
  function! SelectWindowByName(name)
    let num = bufwinnr(a:name)
    if num >= 0 && num != winnr()
      execute 'normal! ' . num . "\<C-W>\<C-W>"
    endif
    return num
  endfunction
endif

" vi:ts=8 sts=2 sw=2 tw=0
