" jcode.vim - Japanese code translater
"
" Maintainer:	Taro Muraoka <koron@tka.att.ne.jp>
" Last change:	16:23:57 19-Feb-2000.
"
" Load a kanji file you cannot read, then type 'gjc'... How fantastic !!!

noremap gjc :call JCodeConvert()<CR>
noremap gjs :call JCodeConvert(1)<CR>
noremap gje :call JCodeConvert(2)<CR>
noremap gjj :call JCodeConvert(3)<CR>

"
" JCodeConvert([outcode])
"   Convert kanji code of current buffer to <outcode> code. Default outcode is
"   specified by system. Win32 is 1:Shift-JIS. Unix is 2:EUC.
"   outcode; 1:Shift-JIS/2:EUC/3:JIS
"
func! JCodeConvert(...)
  let i = 1
  " select output kanji code
  let to = 0
  if a:0 > 0
    let to = a:1
  endif
  if to < 1 || to > 3
    if has('unix')
      let to = 2
    elseif has('win32')
      let to = 1
    else
      echohl ErrorMsg
      echo 'Cannot specify output kanji code'
      echohl None
      return
    endif
  endif
  let from = JCodeGetCode(0,line('$'))
  " convert (select filter)
  if from == 0 || to == from
    echo 'No need to convert'
    " no need to convert nor to output
    return
  endif
  if from == 1
    if to == 2
      echo 'Now converting SJIS to EUC. Wait a moment...'
      call JCode_S2E(0,line('$'))
    else
      echo 'Now converting SJIS to JIS. Wait a moment...'
      call JCode_S2J(0,line('$'))
    endif
  elseif from == 2
    if to == 1
      echo 'Now converting EUC to SJIS. Wait a moment...'
      call JCode_E2S(0,line('$'))
    elseif to == 3
      echo 'Now converting EUC to JIS. Wait a moment...'
      call JCode_E2J(0,line('$'))
    endif
  elseif from == 3
    if to == 1
      echo 'Now converting JIS to SJIS. Wait a moment...'
      call JCode_J2S(0,line('$'))
    elseif to == 2
      echo 'Now converting JIS to EUC. Wait a moment...'
      call JCode_J2E(0,line('$'))
    endif
  endif
endfunc

"
" JCodeGetCode(start, end)
"   Distinguish KANJI code of text range from line start to end.
"   Return 1:Shift-JIS/2:EUC/3:JIS/0:binary(no need to convert)
"
func! JCodeGetCode(start, end)
  let i = a:start
  let sjis = 0
  let euc = 0
  while i < a:end
    let str = getline(i)
    let i = i + 1
    let j = 0
    let len = strlen(str)
    while j < len
      let c = char2nr(str[j])
      let j = j + 1
      if c >= 0x81 && c <= 0x9F && c != 0x8E
	" force Shift-JIS
	return 1
      endif
      if c >= 0x80
	let sjis = sjis + 1
      endif
      if c >= 0xA1 && c <= 0xDF || c == 0x8E
	let euc = euc + 1
      endif
    endw
    if str =~ "\x1B[\x24\x28]"
      " JIS have special sequence
      return 3
    endif
    if euc && sjis / 2 <= euc
      return 2
    endif
  endw
  return 0
endfunc

"
" JCode_?2?(start, end)
"   Area specified by line 'start' from 'end' convert routines.
"
func! JCode_J2S(start, end)
  let i = a:start
  while i <= a:end
    call setline(i, JCode_e2s(JCode_j2e(getline(i))))
    let i = i + 1
  endw
endfunc

func! JCode_S2J(start, end)
  let i = a:start
  while i <= a:end
    call setline(i, JCode_e2j(JCode_s2e(getline(i))))
    let i = i + 1
  endw
endfunc

func! JCode_J2E(start, end)
  let i = a:start
  while i <= a:end
    call setline(i, JCode_j2e(getline(i)))
    let i = i + 1
  endw
endfunc

func! JCode_E2J(start, end)
  let i = a:start
  while i <= a:end
    call setline(i, JCode_e2j(getline(i)))
    let i = i + 1
  endw
endfunc

func! JCode_S2E(start, end)
  let i = a:start
  while i <= a:end
    call setline(i, JCode_s2e(getline(i)))
    let i = i + 1
  endw
endfunc

func! JCode_E2S(start, end)
  let i = a:start
  while i <= a:end
    call setline(i, JCode_e2s(getline(i)))
    let i = i + 1
  endw
endfunc

"
" JCode_getCode(str)
"   Distinguish KANJI code.
"   Return 1:Shift-JIS/2:EUC/3:JIS/0:binary(no need to convert)
"
func! JCode_getCode(str)
  let str = ''
  let len = strlen(a:str)
  let sjis = 0
  let euc = 0
  let esc = 0
  let i = 0
  while i < len
    let c = char2nr(a:str[i])
    "let c = char2nr(strpart(a:str, i, 1))
    let i = i + 1
    if c >= 0x81 && c <= 0x9F && c != 0x8E
      " force Shift-JIS
      return 1
    endif
    if c == 0x1B
      let esc = esc + 1
      continue
    endif
    if c >= 0x80
      let sjis = sjis + 1
    endif
    if c >= 0xA1 && c <= 0xDF || c == 0x8E
      let euc = euc + 1
    endif
  endw
  " make desicion
  if sjis == 0
    if esc > 0
      return 3
    endif
    return 0
  elseif sjis / 2 <= euc
    return 2
  endif
  return 0
endfunc

"
" JCode_j2e(str)
"   Convert str(JIS) to return string(EUC)
"
func! JCode_j2e(str)
  let str = ''
  let len = strlen(a:str)
  let mode = 0
  let i = 0
  while i < len
    " get a byte from string
    let c = char2nr(a:str[i])
    let i = i + 1
    if c != 0x1B
      " normal chracter
      if mode == 1
	let c = c + 0x80
      endif
      " output a byte and next loop
      let str = str . nr2char(c)
      continue
    endif
    if strpart(a:str, i, 2) == "\x24\x42"
      " change to KANJI mode
      let mode = 1
      let i = i + 2
    elseif strpart(a:str, i, 2) =~ "^\x28[\x40-\x7E]$"
      " change to ASCII mode
      let mode = 0
      let i = i + 2
    else
      let str = str . nr2char(c)
      continue
    endif
  endw
  return str
endfunc

"
" JCode_e2j(str)
"   Convert str(EUC) to return string(JIS)
"
func! JCode_e2j(str)
  let str = ''
  let len = strlen(a:str)
  let mode = 0
  let i = 0
  while i < len
    " get a byte from string
    let c = char2nr(a:str[i])
    let i = i + 1
    if mode == 0
      " ASCII mode
      if c >= 0x80
	" change to KANJI mode
	let str = str . "\x1B\x24\x42"
	let c = c - 0x80
	let mode = 1
      endif
    elseif mode == 1
      " KANJI mode
      if c >= 0x80
	let c = c - 0x80
      else
	" change to ASCII mode
	let str = str . "\x1B\x28\x42"
	let mode = 0
      endif
    endif
    let str = str . nr2char(c)
  endw
  " line must be ended ASCII mode
  if mode == 1
    let str = str . "\x1B\x28\x42"
  endif
  return str
endfunc

"
" JCode_s2e(str)
"   Convert str(Shift-JIS) to return string(EUC)
"
func! JCode_s2e(str)
  let str = ''
  let len = strlen(a:str)
  let i = 0
  while i < len
    " parse high byte
    let h = char2nr(a:str[i])
    let i = i + 1
    " is this Shift-JIS (high)?
    if h >= 0x81 && h <= 0x9F
      let h = h * 2 - 0x61
    elseif h >= 0xE0 && h <= 0xEF
      let h = h * 2 - 0xE1
    else
      let str = str . nr2char(h)
      continue
    endif
    " parse low byte
    if i >= len
      break
    endif
    let l = char2nr(a:str[i])
    let i = i + 1
    " is this Shift-JIS (low)?
    if l >= 0x40 && l <= 0x9E
      if l >= 0x80
	let l = l - 1
      endif
      let l = l + 0x61
    elseif l >= 0x9F && l <= 0xFC
      let h = h + 1
      let l = l + 2
    else
      let str = str . nr2char(h) . nr2char(l)
      continue
    endif
    " write EUC
    let str = str . nr2char(h) . nr2char(l)
  endw
  return str
endfunc

"
" JCode_e2s(str)
"   Convert str(EUC) to return string(Shift-JIS)
"
func! JCode_e2s(str)
  let str = ''
  let len = strlen(a:str)
  let i = 0
  while i < len
    " parse high byte
    let h = char2nr(a:str[i])
    let i = i + 1
    " is this EUC?
    if h >= 0xA1 && h <= 0xDD
      let sh = (h + 0x61) / 2
    elseif h >= 0xDE && h <= 0xFE
      let sh = (h + 0xE2) / 2
    else
      let str = str . nr2char(h)
      continue
    endif
    " parse low byte
    if i >= len
      break
    endif
    let l = char2nr(a:str[i])
    let i = i + 1
    " is this EUC (low)?
    if l >= 0xA1 && l <= 0xFE
      if h % 2
	let l = l - 0x61
	if l >= 0x7F
	  let l = l + 1
	endif
      else
	let l = l - 0x02
      endif
    else
      let str = str . nr2char(h) . nr2char(l)
      continue
    endif
    " write Shift-JIS
    let str = str . nr2char(sh) . nr2char(l)
  endw
  return str
endfunc

"
" JCode_convert([outcode])
"   Convert kanji code of current buffer to <outcode> code. Default outcode is
"   specified by system. Win32 is 1:Shift-JIS. Unix is 2:EUC.
"   outcode; 1:Shift-JIS/2:EUC/3:JIS
"   NOTE: This is slow and flexible version.
"
func! JCode_convert(...)
  let i = 1
  " select output kanji code
  let to = 0
  if a:0 > 0
    let to = a:1
  endif
  if to < 1 || to > 3
    if has('unix')
      let to = 2
    elseif has('win32')
      let to = 1
    else
      echohl ErrorMsg
      echo 'Cannot specify output kanji code'
      echohl None
      return
    endif
  endif
  echo 'Now converting. Wait a moment...'
  " convert
  while i <= line('$')
    " get line string
    let str = getline(i)
    " distinguish kanji code
    let from = JCode_getCode(str)
    " select filter
    if from == 0 || to == from
      " no need to convert nor to output
      let i = i + 1
      continue
    elseif from == 1
      " SJIS to EUC
      let str = JCode_s2e(str)
      if to == 3
	" SJIS to JIS
	let str = JCode_e2j(str)
      endif
    elseif from == 2
      if to == 1
	" EUC to SJIS
	let str = JCode_e2s(str)
      elseif to == 3
	" EUC to JIS
	let str = JCode_e2j(str)
      endif
    elseif from == 3
      " JIS to EUC
      let str = JCode_j2e(str)
      if to == 1
	" JIS to SJIS
	let str = JCode_e2s(str)
      endif
    endif
    " output string
    call setline(i, str)
    let i = i + 1
  endw
endfunc

" vi:ts=8 sts=2 sw=2 tw=0:
